package com.chelvc.framework.database.handler;

import java.io.IOException;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;
import java.util.Set;

import com.chelvc.framework.common.annotation.Sensitive;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.introspect.Annotated;
import com.fasterxml.jackson.databind.introspect.JacksonAnnotationIntrospector;
import com.fasterxml.jackson.databind.type.CollectionType;
import lombok.NonNull;
import org.apache.ibatis.executor.result.ResultMapException;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeException;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.TypeReference;

/**
 * JSON类型处理器接口
 *
 * @param <T> 对象类型
 * @author Woody
 * @date 2024/4/3
 */
public interface JsonTypeHandler<T> extends TypeHandler<T> {
    /**
     * 敏感字符串序列化处理器
     */
    class SensitiveSerializer extends JsonSerializer<String> {
        @Override
        public void serialize(String value, JsonGenerator generator, SerializerProvider provider) throws IOException {
            if (DatabaseContextHolder.isSensitiveWritable() && value != null) {
                value = AESUtils.encrypt(DatabaseContextHolder.getEncryptor(), value);
            }
            generator.writeString(value);
        }
    }

    /**
     * 敏感字符串数组序列化处理器
     */
    class SensitiveArraySerializer extends JsonSerializer<String[]> {
        @Override
        public void serialize(String[] value, JsonGenerator generator, SerializerProvider provider)
                throws IOException {
            if (DatabaseContextHolder.isSensitiveWritable() && ObjectUtils.notEmpty(value)) {
                value = AESUtils.encrypt(DatabaseContextHolder.getEncryptor(), value);
            }
            generator.writeObject(value);
        }
    }

    /**
     * 敏感字符串集合序列化处理器
     */
    class SensitiveSetSerializer extends JsonSerializer<Set<String>> {
        @Override
        public void serialize(Set<String> value, JsonGenerator generator, SerializerProvider provider)
                throws IOException {
            if (DatabaseContextHolder.isSensitiveWritable() && ObjectUtils.notEmpty(value)) {
                value = AESUtils.encrypt(DatabaseContextHolder.getEncryptor(), value);
            }
            generator.writeObject(value);
        }
    }

    /**
     * 敏感字符串列表序列化处理器
     */
    class SensitiveListSerializer extends JsonSerializer<List<String>> {
        @Override
        public void serialize(List<String> value, JsonGenerator generator, SerializerProvider provider)
                throws IOException {
            if (DatabaseContextHolder.isSensitiveWritable() && ObjectUtils.notEmpty(value)) {
                value = AESUtils.encrypt(DatabaseContextHolder.getEncryptor(), value);
            }
            generator.writeObject(value);
        }
    }

    /**
     * 敏感字符串反序列化处理器
     */
    class SensitiveDeserializer extends JsonDeserializer<String> {
        @Override
        public String deserialize(JsonParser parser, DeserializationContext context)
                throws IOException, JsonProcessingException {
            String value = parser.getValueAsString();
            return value == null ? null : AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), value);
        }
    }

    /**
     * 敏感字符串数组反序列化处理器
     */
    class SensitiveArrayDeserializer extends JsonDeserializer<String[]> {
        private static final com.fasterxml.jackson.core.type.TypeReference<String[]> TYPE =
                new com.fasterxml.jackson.core.type.TypeReference<String[]>() {
                };

        @Override
        public String[] deserialize(JsonParser parser, DeserializationContext context)
                throws IOException, JsonProcessingException {
            String[] values = parser.readValueAs(TYPE);
            if (ObjectUtils.isEmpty(values)) {
                return values;
            }
            return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
        }
    }

    /**
     * 敏感字符串集合反序列化处理器
     */
    class SensitiveSetDeserializer extends JsonDeserializer<Set<String>> {
        private static final com.fasterxml.jackson.core.type.TypeReference<Set<String>> TYPE =
                new com.fasterxml.jackson.core.type.TypeReference<Set<String>>() {
                };

        @Override
        public Set<String> deserialize(JsonParser parser, DeserializationContext context)
                throws IOException, JsonProcessingException {
            Set<String> values = parser.readValueAs(TYPE);
            if (ObjectUtils.isEmpty(values)) {
                return values;
            }
            return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
        }
    }

    /**
     * 敏感字符串列表反序列化处理器
     */
    class SensitiveListDeserializer extends JsonDeserializer<List<String>> {
        private static final com.fasterxml.jackson.core.type.TypeReference<List<String>> TYPE =
                new com.fasterxml.jackson.core.type.TypeReference<List<String>>() {
                };

        @Override
        public List<String> deserialize(JsonParser parser, DeserializationContext context)
                throws IOException, JsonProcessingException {
            List<String> values = parser.readValueAs(TYPE);
            if (ObjectUtils.isEmpty(values)) {
                return values;
            }
            return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
        }
    }

    /**
     * 对象JSON序列化/反序列化处理器实例
     */
    ObjectMapper mapper = JacksonUtils.initializeSerializer().setAnnotationIntrospector(
            new JacksonAnnotationIntrospector() {
                @Override
                public Object findSerializer(Annotated am) {
                    if (!am.hasAnnotation(Sensitive.class)) {
                        return super.findSerializer(am);
                    }
                    JavaType type = am.getType();
                    Class<?> raw = am.getRawType();
                    if (raw == String.class) {
                        return SensitiveSerializer.class;
                    } else if (raw.isArray() && raw.getComponentType() == String.class) {
                        return SensitiveArraySerializer.class;
                    } else if (type instanceof CollectionType && type.getContentType().getRawClass() == String.class) {
                        if (Set.class.isAssignableFrom(type.getRawClass())) {
                            return SensitiveSetSerializer.class;
                        }
                        return SensitiveListSerializer.class;
                    }
                    return super.findSerializer(am);
                }

                @Override
                public Object findDeserializer(Annotated am) {
                    if (!am.hasAnnotation(Sensitive.class)) {
                        return super.findDeserializer(am);
                    }
                    AnnotatedElement element = am.getAnnotated();
                    if (element instanceof Method) {
                        Parameter[] parameters = ((Method) element).getParameters();
                        if (ObjectUtils.notEmpty(parameters) && parameters.length > 0) {
                            Type type = parameters[0].getParameterizedType();
                            if (type == String.class) {
                                return SensitiveDeserializer.class;
                            } else if (type instanceof ParameterizedType) {
                                Type raw = ((ParameterizedType) type).getRawType();
                                Type arg = ((ParameterizedType) type).getActualTypeArguments()[0];
                                if (raw == Set.class && arg == String.class) {
                                    return SensitiveSetDeserializer.class;
                                } else if (raw == List.class && arg == String.class) {
                                    return SensitiveListDeserializer.class;
                                }
                            } else if (type instanceof Class<?> && ((Class<?>) type).isArray()
                                    && ((Class<?>) type).getComponentType() == String.class) {
                                return SensitiveArrayDeserializer.class;
                            }
                        }
                    } else {
                        JavaType type = am.getType();
                        Class<?> raw = am.getRawType();
                        if (raw == String.class) {
                            return SensitiveDeserializer.class;
                        } else if (raw.isArray() && raw.getComponentType() == String.class) {
                            return SensitiveArrayDeserializer.class;
                        } else if (type instanceof CollectionType
                                && type.getContentType().getRawClass() == String.class) {
                            if (Set.class.isAssignableFrom(type.getRawClass())) {
                                return SensitiveSetDeserializer.class;
                            }
                            return SensitiveListDeserializer.class;
                        }
                    }
                    return super.findDeserializer(am);
                }
            });

    /**
     * 获取对象类型
     *
     * @return 对象类型
     */
    Type getRawType();

    @Override
    default void setParameter(PreparedStatement ps, int i, T parameter, JdbcType jdbcType) throws SQLException {
        if (parameter == null) {
            if (jdbcType == null) {
                throw new TypeException("JDBC requires that the JdbcType must be specified " +
                        "for all nullable parameters.");
            }
            try {
                ps.setNull(i, jdbcType.TYPE_CODE);
            } catch (SQLException e) {
                throw new TypeException("Error setting null for parameter #" + i + " with JdbcType " +
                        jdbcType + " . Try setting a different JdbcType for this parameter " +
                        "or a different jdbcTypeForNull configuration property. Cause: " + e, e);
            }
        } else {
            try {
                setNonNullParameter(ps, i, parameter, jdbcType);
            } catch (Exception e) {
                throw new TypeException("Error setting non null for parameter #" + i + " with JdbcType " +
                        jdbcType + " . Try setting a different JdbcType for this parameter " +
                        "or a different configuration property. Cause: " + e, e);
            }
        }
    }

    @Override
    default T getResult(ResultSet rs, String columnName) throws SQLException {
        try {
            return getNullableResult(rs, columnName);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column '" + columnName + "' from result set.  " +
                    "Cause: " + e, e);
        }
    }

    @Override
    default T getResult(ResultSet rs, int columnIndex) throws SQLException {
        try {
            return getNullableResult(rs, columnIndex);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column #" + columnIndex + " from result set.  " +
                    "Cause: " + e, e);
        }
    }

    @Override
    default T getResult(CallableStatement cs, int columnIndex) throws SQLException {
        try {
            return getNullableResult(cs, columnIndex);
        } catch (Exception e) {
            throw new ResultMapException("Error attempting to get column #" + columnIndex + " from callable statement" +
                    ".  Cause: " + e, e);
        }
    }

    /**
     * 设置参数值
     *
     * @param ps        PreparedStatement对象实例
     * @param i         参数下标
     * @param parameter 参数值
     * @param jdbcType  数据库字段类型
     * @throws SQLException SQL异常
     */
    default void setNonNullParameter(PreparedStatement ps, int i, T parameter, JdbcType jdbcType) throws SQLException {
        if (ObjectUtils.isBlank(parameter)) {
            ps.setNull(i, Types.VARCHAR);
        } else {
            ps.setString(i, JacksonUtils.serialize(mapper, parameter));
        }
    }

    /**
     * 获取字段结果
     *
     * @param rs         结果集合对象实例
     * @param columnName 字段名称
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(ResultSet rs, String columnName) throws SQLException {
        return (T) JacksonUtils.deserialize(mapper, rs.getString(columnName), this.getRawType());
    }

    /**
     * 获取字段结果
     *
     * @param rs          结果集合对象实例
     * @param columnIndex 字段下标
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        return (T) JacksonUtils.deserialize(mapper, rs.getString(columnIndex), this.getRawType());
    }

    /**
     * 获取字段结果
     *
     * @param cs          CallableStatement对象实例
     * @param columnIndex 字段下标
     * @return 字段值
     * @throws SQLException SQL异常
     */
    @SuppressWarnings("unchecked")
    default T getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        return (T) JacksonUtils.deserialize(mapper, cs.getString(columnIndex), this.getRawType());
    }

    /**
     * JSON类型处理器默认实现
     *
     * @param <T> 对象类型
     */
    class Default<T> extends TypeReference<T> implements JsonTypeHandler<T> {
    }

    /**
     * JSON类型处理器简单实现
     *
     * @param <T> 对象类型
     */
    class Simple<T> implements JsonTypeHandler<T> {
        private final Type type;

        public Simple(@NonNull Type type) {
            this.type = type;
        }

        @Override
        public Type getRawType() {
            return this.type;
        }
    }
}
