package com.chelvc.framework.database.support;

import java.lang.reflect.Type;
import java.util.List;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.QueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.listener.AddListener;
import com.chelvc.framework.database.listener.DeleteListener;
import com.chelvc.framework.database.listener.ModifyListener;
import com.chelvc.framework.database.listener.OperateListener;
import com.google.common.collect.Lists;
import lombok.NonNull;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

/**
 * 业务增强操作默认实现
 *
 * @param <T> 数据模型类型
 * @param <M> 数据Mapper类型
 * @author Woody
 * @date 2024/4/29
 */
public class DefaultEnhanceService<M extends BaseMapper<T>, T> extends ServiceImpl<M, T>
        implements EnhanceService<T>, ApplicationContextAware {
    private final List<AddListener<T>> addListeners = Lists.newLinkedList();
    private final List<DeleteListener<T>> deleteListeners = Lists.newLinkedList();
    private final List<ModifyListener<T>> modifyListeners = Lists.newLinkedList();

    /**
     * 判断是否是当前数据模型监听器
     *
     * @param type     操作类型
     * @param listener 监听器实例
     * @return true/false
     */
    private boolean isEntityListener(Class<?> type, OperateListener<?> listener) {
        Type parameterized = ObjectUtils.lookupSuperclassParameterized(listener.getClass(), type);
        return ObjectUtils.type2class(parameterized) == this.getEntityClass();
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        // 注册事件监听器
        applicationContext.getBeansOfType(OperateListener.class).values().forEach(this::register);
    }

    @Override
    public QueryChainWrapper<T> query() {
        QueryChainWrapper<T> query = super.query();
        query.setEntityClass(this.getEntityClass());
        return query;
    }

    @Override
    public LambdaQueryChainWrapper<T> lambdaQuery() {
        LambdaQueryChainWrapper<T> query = super.lambdaQuery();
        query.setEntityClass(this.getEntityClass());
        return query;
    }

    @Override
    public synchronized void register(@NonNull OperateListener<T> listener) {
        if (listener instanceof AddListener && this.isEntityListener(AddListener.class, listener)) {
            this.addListeners.add((AddListener<T>) listener);
        }
        if (listener instanceof DeleteListener && this.isEntityListener(DeleteListener.class, listener)) {
            this.deleteListeners.add((DeleteListener<T>) listener);
        }
        if (listener instanceof ModifyListener && this.isEntityListener(ModifyListener.class, listener)) {
            this.modifyListeners.add((ModifyListener<T>) listener);
        }
    }

    @Override
    public void publishAddition(@NonNull T entity) {
        this.addListeners.forEach(listener -> listener.add(entity));
    }

    @Override
    public void publishDeletion(@NonNull T entity) {
        this.deleteListeners.forEach(listener -> listener.delete(entity));
    }

    @Override
    public void publishModification(@NonNull T before, @NonNull T after) {
        this.modifyListeners.forEach(listener -> listener.modify(before, after));
    }
}
