package com.chelvc.framework.security.context;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.NonNull;

/**
 * 应用安全相关操作上下文工具类
 *
 * @author Woody
 * @date 2024/5/24
 */
public final class SecurityContextHolder {
    /**
     * 默认权限分组
     */
    public static final String DEFAULT_PERMISSION_GROUP = "DEFAULT";

    /**
     * 配置属性
     */
    private static SecurityProperties PROPERTIES;

    private SecurityContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性
     */
    private static SecurityProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (SecurityProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(SecurityProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取密码处理器
     *
     * @param mode 加解密模式
     * @return 密码处理器实例
     */
    private static Cipher getCipher(int mode) {
        SecurityProperties properties = getProperties();
        String secret = AssertUtils.nonempty(properties.getSecret(), () -> "Cipher secret is missing");
        String iv = AssertUtils.nonempty(properties.getIv(), () -> "Cipher iv is missing");
        return AESUtils.lookupCipher(
                AESUtils.CBC_PKCS5PADDING + secret, mode,
                () -> AESUtils.getCipher(AESUtils.CBC_PKCS5PADDING, mode, secret, iv)
        );
    }

    /**
     * 获取加密处理器
     *
     * @return 加密处理器
     */
    public static Cipher getEncryptor() {
        return getCipher(Cipher.ENCRYPT_MODE);
    }

    /**
     * 获取解密处理器
     *
     * @return 解密处理器
     */
    public static Cipher getDecrypter() {
        return getCipher(Cipher.DECRYPT_MODE);
    }

    /**
     * 标记敏感信息
     *
     * @param ciphertext 敏感信息密文
     * @return 敏感信息
     */
    public static String mark(String ciphertext) {
        SecurityProperties.Mark mark = getProperties().getMark();
        if (!mark.isEnabled() || StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }
        String prefix = ObjectUtils.ifNull(mark.getPrefix(), StringUtils.EMPTY);
        String suffix = ObjectUtils.ifNull(mark.getSuffix(), StringUtils.EMPTY);
        return prefix + ciphertext + suffix;
    }

    /**
     * 标记敏感信息
     *
     * @param ciphertexts 敏感信息密文数组
     * @return 敏感信息数组
     */
    public static String[] mark(String... ciphertexts) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        String[] marks = new String[ciphertexts.length];
        for (int i = 0; i < ciphertexts.length; i++) {
            marks[i] = mark(ciphertexts[i]);
        }
        return marks;
    }

    /**
     * 标记敏感信息
     *
     * @param ciphertexts 敏感信息密文集合
     * @return 敏感信息集合
     */
    public static Set<String> mark(Set<String> ciphertexts) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        return ciphertexts.stream().map(SecurityContextHolder::mark).collect(Collectors.toSet());
    }

    /**
     * 标记敏感信息
     *
     * @param ciphertexts 敏感信息密文列表
     * @return 敏感信息列表
     */
    public static List<String> mark(List<String> ciphertexts) {
        if (ObjectUtils.isEmpty(ciphertexts)) {
            return ciphertexts;
        }
        return ciphertexts.stream().map(SecurityContextHolder::mark).collect(Collectors.toList());
    }

    /**
     * 数字签名
     *
     * @param session 会话信息
     * @param payload 请求数据
     * @return 签名信息
     */
    public static String sign(@NonNull Session session, String payload) {
        String secret = AssertUtils.nonempty(getProperties().getSecret(), () -> "secret is missing");
        String plaintext = secret + session.getPlatform() + session.getTerminal() + session.getVersion() +
                session.getTimestamp() + payload;
        return CodecUtils.md5(plaintext);
    }

    /**
     * 校验请求时间戳是否有效
     *
     * @param timestamp 时间戳
     * @return true/false
     */
    public static boolean validateTimestamp(long timestamp) {
        long duration = getProperties().getDuration();
        return duration < 1 || Math.abs(System.currentTimeMillis() - timestamp) <= duration;
    }

    /**
     * 判断当前用户是否拥有任意权限
     *
     * @param permissions 权限标识数组
     * @return true/false
     */
    public static boolean hashAnyPermission(@NonNull String... permissions) {
        if (ObjectUtils.isEmpty(permissions)) {
            return true;
        }
        Session session = SessionContextHolder.getSession(false);
        Set<String> authorities = ObjectUtils.ifNull(session, Session::getAuthorities);
        if (ObjectUtils.notEmpty(authorities)) {
            Stream<String> stream = Stream.of(permissions);
            if (stream.anyMatch(authorities::contains)) {
                return true;
            }
            for (String authority : authorities) {
                Set<?> properties = ApplicationContextHolder.getProperty(authority, Set.class);
                if (ObjectUtils.notEmpty(properties) && stream.anyMatch(properties::contains)) {
                    return true;
                }
            }
        }
        return false;
    }
}
