package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.base.interceptor.ResponseHandler;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.core.MethodParameter;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdviceAdapter;

/**
 * 接口参数加解密拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@ControllerAdvice
@Order(Ordered.LOWEST_PRECEDENCE - 1)
public class ControllerCryptoInterceptor extends RequestBodyAdviceAdapter implements ResponseHandler {
    /**
     * 获取目标方法@Crypto注解
     *
     * @param method 目标方法
     * @return @Crypto注解实例
     */
    private Crypto getCryptoAnnotation(MethodParameter method) {
        Crypto annotation = method.getMethodAnnotation(Crypto.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(Crypto.class);
        }
        return annotation;
    }

    /**
     * 判断是否是输入加密方法
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isInputCryptoMethod(MethodParameter method) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.input();
    }

    /**
     * 判断是否是输出加密方法
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isOutputCryptoMethod(MethodParameter method) {
        Crypto annotation = this.getCryptoAnnotation(method);
        return annotation != null && annotation.output();
    }

    /**
     * 获取消息密文
     *
     * @param message 消息对象
     * @return 密文字节数组
     * @throws IOException I/O异常
     */
    private byte[] getCiphertext(HttpInputMessage message) throws IOException {
        InputStream input = message.getBody();
        if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
            return ((BufferedRequestWrapper.BufferedServletInputStream) input).decodeBase64();
        }
        return Base64.decodeBase64(FileUtils.getBytes(input));
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        return this.isInputCryptoMethod(method);
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        Cipher cipher = SecurityContextHolder.getDecrypter();
        byte[] plaintext, ciphertext = this.getCiphertext(message);
        try {
            plaintext = AESUtils.codec(cipher, ciphertext);
        } catch (Exception e) {
            log.warn("Request parameter decrypt failed: {}", e.getMessage());
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Request.Invalid"));
        }
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(plaintext);
            }
        };
    }

    @Override
    public Object handle(NativeWebRequest request, MethodParameter method, Object value) {
        if (value != null && this.isOutputCryptoMethod(method)) {
            Cipher cipher = SecurityContextHolder.getEncryptor();
            value = AESUtils.encrypt(cipher, JacksonContextHolder.serialize(value));
        }
        return value;
    }
}
