package com.chelvc.framework.security.interceptor;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.security.annotation.Permission;
import com.chelvc.framework.security.context.SecurityContextHolder;
import com.google.common.collect.Sets;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationListener;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 自定义权限校验拦截器
 *
 * @author Woody
 * @date 2024/5/19
 */
@Component
public class PermissionValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer,
        ApplicationListener<ApplicationStartedEvent> {
    /**
     * 获取接口权限注解实例
     *
     * @param handler 接口处理器实例
     * @return 接口权限注解实例
     */
    private Permission getPermissionAnnotation(Object handler) {
        if (!(handler instanceof HandlerMethod)) {
            return null;
        }
        return ((HandlerMethod) handler).getMethodAnnotation(Permission.class);
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        // 检查权限标识是否重复
        ApplicationContext applicationContext = event.getApplicationContext();
        List<Object> controllers = ApplicationContextHolder.lookupControllers(applicationContext);
        if (ObjectUtils.isEmpty(controllers)) {
            return;
        }
        Set<String> permissions = Sets.newHashSet();
        controllers.forEach(controller -> {
            Class<?> clazz = AopProxyUtils.ultimateTargetClass(controller);
            for (Method method : clazz.getDeclaredMethods()) {
                Permission annotation = method.getAnnotation(Permission.class);
                if (annotation == null) {
                    continue;
                }
                String value = annotation.value(), group = annotation.group();
                AssertUtils.nonempty(value, () -> "Permission value must not be empty: " + method);
                AssertUtils.nonempty(group, () -> "Permission group must not be empty: " + method);
                AssertUtils.check(permissions.add(value), () -> "Duplicate permission value: " + value);
            }
        });
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        Permission annotation = this.getPermissionAnnotation(handler);
        if (annotation != null &&
                !SecurityContextHolder.hashAnyPermission(annotation.value(), annotation.group())) {
            throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null,
                    ApplicationContextHolder.getMessage("Forbidden"));
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE + 1);
    }
}
