package com.chelvc.framework.security.interceptor;

import java.util.Objects;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Security;
import com.chelvc.framework.security.context.SecurityContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
public class SecurityValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    /**
     * 签名信息请求头
     */
    private static final String HEADER_SIGNATURE = "signature";

    /**
     * 获取接口安全注解实例
     *
     * @param handler 接口处理器实例
     * @return 接口安全注解实例
     */
    private Security getSecurityAnnotation(Object handler) {
        if (!(handler instanceof HandlerMethod)) {
            return null;
        }
        HandlerMethod method = (HandlerMethod) handler;
        Security annotation = method.getMethodAnnotation(Security.class);
        if (annotation == null) {
            return method.getMethod().getDeclaringClass().getAnnotation(Security.class);
        }
        return annotation;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        Security annotation = this.getSecurityAnnotation(handler);
        if (annotation == null || !annotation.enabled() || !(annotation.sign() || annotation.header())) {
            return true;
        }

        // 请求头校验
        Session session = SessionContextHolder.getSession();
        if (annotation.header()) {
            // 非空请求头校验
            if (session.getPlatform() == null || session.getTerminal() == null
                    || StringUtils.isEmpty(session.getVersion()) || session.getTimestamp() == null) {
                throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                        ApplicationContextHolder.getMessage("Header.Missing"));
            }

            // 请求时间戳校验
            if (!SecurityContextHolder.validateTimestamp(session.getTimestamp())) {
                throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                        ApplicationContextHolder.getMessage("Time.Deviated"));
            }
        }

        // 数字签名校验
        if (annotation.sign()) {
            String payload = HttpUtils.serialize(request, true);
            String signature = request.getHeader(HEADER_SIGNATURE);
            String ciphertext = SecurityContextHolder.sign(session, payload);
            LoggingContextHolder.debug(log, signature, ciphertext, payload);
            if (!Objects.equals(ciphertext, signature)) {
                throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                        ApplicationContextHolder.getMessage("Signature.Invalid"));
            }
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE);
    }
}
