package com.chelvc.framework.security.interceptor;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.crypto.Cipher;

import com.chelvc.framework.base.interceptor.ResponseHandler;
import com.chelvc.framework.common.annotation.Sensitive;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.security.context.SecurityContextHolder;
import org.springframework.core.MethodParameter;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;

/**
 * 敏感数据响应拦截器实现
 *
 * @author Woody
 * @date 2024/6/22
 */
@Component
@Order(Ordered.LOWEST_PRECEDENCE - 2)
public class SensitiveResponseInterceptor implements ResponseHandler {
    /**
     * 判断是否是敏感数据方法
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isSensitiveMethod(MethodParameter method) {
        return method.hasMethodAnnotation(Sensitive.class);
    }

    @Override
    @SuppressWarnings("unchecked")
    public Object handle(NativeWebRequest request, MethodParameter method, Object value) {
        if (value == null || !this.isSensitiveMethod(method)) {
            return value;
        }
        Cipher cipher = SecurityContextHolder.getEncryptor();
        Type type = Objects.requireNonNull(method.getMethod()).getGenericReturnType();
        if (type == String.class) {
            value = SecurityContextHolder.mark(AESUtils.encrypt(cipher, (String) value));
        } else if (type instanceof ParameterizedType && ObjectUtils.notBlank(value)) {
            Type raw = ((ParameterizedType) type).getRawType();
            Type arg = ((ParameterizedType) type).getActualTypeArguments()[0];
            if (raw == Set.class && arg == String.class) {
                value = SecurityContextHolder.mark(AESUtils.encrypt(cipher, (Set<String>) value));
            } else if (raw == List.class && arg == String.class) {
                value = SecurityContextHolder.mark(AESUtils.encrypt(cipher, (List<String>) value));
            }
        } else if (type instanceof Class && ((Class<?>) type).isArray()
                && ((Class<?>) type).getComponentType() == String.class && ObjectUtils.notBlank(value)) {
            value = SecurityContextHolder.mark(AESUtils.encrypt(cipher, (String[]) value));
        }
        return value;
    }
}
