package com.chelvc.framework.upload.support;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.upload.UploadHandler;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 文件上传处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingUploadHandler implements UploadHandler {
    private final List<UploadHandler> handlers;

    public DelegatingUploadHandler(@NonNull List<UploadHandler> handlers) {
        this.handlers = AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
    }

    /**
     * 文件上传执行器接口
     */
    private interface Executor {
        /**
         * 执行文件上传处理
         *
         * @param handler 文件上传处理器
         * @return 文件路径
         * @throws IOException I/O异常
         */
        String execute(UploadHandler handler) throws IOException;
    }

    /**
     * 遍历所有短信处理器并执行文件上传处理，直到上传成功
     *
     * @param executor 文件上传执行器
     */
    private String execute(Executor executor) throws IOException {
        RuntimeException unsupported = null;
        for (int i = 0, last = this.handlers.size() - 1; i <= last; i++) {
            try {
                return executor.execute(this.handlers.get(i));
            } catch (Exception e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = (UnsupportedOperationException) e;
                } else if (i < last) {
                    log.warn("File upload failed: {}", e.getMessage());
                } else {
                    throw e;
                }
            }
        }
        if (unsupported != null) {
            throw unsupported;
        }
        return null;
    }

    @Override
    public void initialize() {
        this.handlers.forEach(UploadHandler::initialize);
    }

    @Override
    public String upload(@NonNull File file) throws IOException {
        return this.execute(handler -> handler.upload(file));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix) throws IOException {
        return this.execute(handler -> handler.upload(stream, suffix));
    }

    @Override
    public String upload(@NonNull InputStream stream, String suffix, long length) throws IOException {
        return this.execute(handler -> handler.upload(stream, suffix, length));
    }

    @Override
    public void destroy() {
        this.handlers.forEach(UploadHandler::destroy);
    }
}
