package com.chelvc.framework.wechat;

import java.time.Duration;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.chelvc.framework.wechat.context.WechatContextHolder;
import com.google.common.collect.ImmutableMap;
import lombok.NonNull;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;

/**
 * 微信小程序操作默认实现
 *
 * @author Woody
 * @date 2024/6/19
 */
public class DefaultWechatAppletHandler implements WechatAppletHandler {
    /**
     * Scheme过期时间（天）
     */
    private static final int SCHEME_EXPIRE_INTERVAL = 30;

    /**
     * 获取用户信息接口地址
     */
    private static final String USER_URL =
            "https://api.weixin.qq.com/cgi-bin/user/info?access_token=%s&openid=%s&lang=zh_CN";

    /**
     * 微信获取手机号接口地址
     */
    private static final String CODE2MOBILE_URL =
            "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=";

    /**
     * 微信登录接口地址
     */
    private static final String JSCODE2SESSION_URL =
            "https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code";

    /**
     * 获取小程序schemeCode接口地址
     */
    private static final String GENERATE_SCHEME_URL = "https://api.weixin.qq.com/wxa/generatescheme?access_token=";

    private final WechatProperties.Applet properties;

    public DefaultWechatAppletHandler(@NonNull WechatProperties.Applet properties) {
        this.properties = properties;
    }

    @Override
    public String getId() {
        return this.properties.getId();
    }

    @Override
    public String getScheme(String path, String query, String environment) {
        // 尝试从Redis中获取Scheme
        path = StringUtils.ifEmpty(path, StringUtils.EMPTY);
        query = StringUtils.ifEmpty(query, StringUtils.EMPTY);
        environment = StringUtils.ifEmpty(environment, StringUtils.EMPTY);
        HttpSession session = ObjectUtils.ifNull(SessionContextHolder.getRequest(), HttpServletRequest::getSession);
        String id = ObjectUtils.ifNull(
                ObjectUtils.ifNull(session, HttpSession::getId), () -> UUID.randomUUID().toString()
        );
        String key = "wechat:scheme:" + id + ":" + environment + ":" + path + ":" + query;
        String scheme = (String) RedisContextHolder.getDefaultTemplate().opsForValue().get(key);
        if (StringUtils.notEmpty(scheme)) {
            return scheme;
        }

        // 重新获取Scheme
        Map<String, Object> body = ImmutableMap.of(
                "jump_wxa", ImmutableMap.of(
                        "path", path,
                        "query", query,
                        "env_version", environment
                ),
                "expire_type", 1,
                "expire_interval", SCHEME_EXPIRE_INTERVAL
        );
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<?> entity = new HttpEntity<>(body, headers);
        String url = GENERATE_SCHEME_URL + this.getAccessToken();
        scheme = WechatContextHolder.exchange(url, HttpMethod.POST, entity, WechatScheme.class).getOpenlink();
        Duration duration = Duration.ofDays(SCHEME_EXPIRE_INTERVAL - 1);
        RedisContextHolder.getDefaultTemplate().opsForValue().set(key, scheme, duration);
        return scheme;
    }

    @Override
    public String getAccessToken() {
        return WechatContextHolder.getAccessToken(this.properties.getAppid());
    }

    @Override
    public WechatUser getUser(@NonNull String openid) {
        String url = String.format(USER_URL, this.getAccessToken(), openid);
        return WechatContextHolder.get(url, WechatUser.class);
    }

    @Override
    public String code2mobile(@NonNull String code) {
        String url = CODE2MOBILE_URL + this.getAccessToken();
        Map<String, String> request = ImmutableMap.of("code", code);
        WechatMobile.Mobile mobile = WechatContextHolder.post(url, request, WechatMobile.class).getMobile();
        return Objects.requireNonNull(mobile).getNumber();
    }

    @Override
    public WechatSession code2session(@NonNull String code) {
        String url = String.format(JSCODE2SESSION_URL, this.properties.getAppid(), this.properties.getSecret(), code);
        return RestContextHolder.get(url, WechatSession.class);
    }
}
