package com.chelvc.framework.wechat.context;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.RestContextHolder;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.wechat.WechatAccessToken;
import com.chelvc.framework.wechat.WechatPublicHandler;
import com.chelvc.framework.wechat.WechatResponse;
import com.chelvc.framework.wechat.config.WechatProperties;
import com.github.wxpay.sdk.WXPayUtil;
import com.google.common.collect.Maps;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * 微信上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
public final class WechatContextHolder implements ApplicationListener<ApplicationStartedEvent> {
    /**
     * 获取访问令牌接口地址
     */
    private static final String ACCESS_TOKEN_URL =
            "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s";

    /**
     * 应用标识/令牌映射表
     */
    private static final Map<String, String> APP_TOKEN_MAPPING = Maps.newConcurrentMap();

    /**
     * 微信公众号处理器映射表
     */
    private static Map<String, WechatPublicHandler> WECHAT_PUBLIC_HANDLER_MAPPING;

    /**
     * 工具类初始化标记
     */
    private volatile boolean initialized;

    private WechatContextHolder() {
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        if (!this.initialized) {
            synchronized (this) {
                if (!this.initialized) {
                    initialize(ApplicationContextHolder.getBean(WechatProperties.class));
                    this.initialized = true;
                }
            }
        }
    }

    /**
     * 初始化
     *
     * @param properties 微信配置属性
     */
    private static void initialize(WechatProperties properties) {
        if (!properties.getToken().isRefreshable() || (ObjectUtils.isEmpty(properties.getApplets())
                && ObjectUtils.isEmpty(properties.getPublics()))) {
            return;
        }

        // 初始化令牌刷新服务线程
        ThreadUtils.run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 刷新小程序访问令牌
                if (ObjectUtils.notEmpty(properties.getApplets())) {
                    properties.getApplets().forEach(config -> refreshAccessToken(
                            config.getAppid(), config.getSecret()
                    ));
                }

                // 刷新公众号访问令牌
                if (ObjectUtils.notEmpty(properties.getPublics())) {
                    properties.getPublics().forEach(config -> refreshAccessToken(
                            config.getAppid(), config.getSecret()
                    ));
                }

                ThreadUtils.sleep(60000);
            }
        });
    }

    /**
     * 刷新访问令牌
     *
     * @param appid  AppID
     * @param secret AppSecret
     */
    private static void refreshAccessToken(String appid, String secret) {
        String key = "wechat:token:" + appid;
        try {
            RedisContextHolder.tryLockAround(key + ":lock", () -> {
                // 获取令牌当前有效期（秒）
                Long ttl = RedisContextHolder.execute(
                        RedisContextHolder.getDefaultConnectionFactory(),
                        connection -> connection.ttl(key.getBytes(StandardCharsets.UTF_8))
                );

                // 如果令牌有效期小于120秒则刷新令牌
                if (ttl == null || ttl < 120) {
                    String url = String.format(ACCESS_TOKEN_URL, appid, secret);
                    WechatAccessToken accessToken = get(url, WechatAccessToken.class);
                    String token = accessToken.getToken();
                    long duration = accessToken.getDuration();
                    RedisContextHolder.getDefaultTemplate().opsForValue().set(key, token, duration, TimeUnit.SECONDS);
                    APP_TOKEN_MAPPING.put(appid, token);
                    if (log.isDebugEnabled()) {
                        log.debug("Wechat access token refreshed: {}, {}", appid, token);
                    }
                }
            });
        } catch (Exception e) {
            log.warn("Wechat access token refresh failed: {}", e.getMessage());
        }
    }

    /**
     * 获取微信访问令牌
     *
     * @param appid AppID
     * @return 访问令牌
     */
    public static String getAccessToken(@NonNull String appid) {
        return APP_TOKEN_MAPPING.computeIfAbsent(appid, k -> {
            String key = "wechat:token:" + appid;
            String token = (String) RedisContextHolder.getDefaultTemplate().opsForValue().get(key);
            if (StringUtils.isEmpty(token)) {
                throw new IllegalStateException("Wechat access token has not been initialized: " + appid);
            }
            return token;
        });
    }

    /**
     * 获取微信公众号处理器
     *
     * @return 处理器名称/实例映射表
     */
    public static Map<String, WechatPublicHandler> getWechatPublicHandlers() {
        if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
            synchronized (WechatPublicHandler.class) {
                if (WECHAT_PUBLIC_HANDLER_MAPPING == null) {
                    Map<String, WechatPublicHandler> handlers =
                            ApplicationContextHolder.getBeans(WechatPublicHandler.class);
                    if (ObjectUtils.isEmpty(handlers)) {
                        WECHAT_PUBLIC_HANDLER_MAPPING = Collections.emptyMap();
                    } else {
                        WECHAT_PUBLIC_HANDLER_MAPPING = Collections.unmodifiableMap(Maps.newHashMap(handlers));
                    }
                }
            }
        }
        return WECHAT_PUBLIC_HANDLER_MAPPING;
    }

    /**
     * 获取微信公众号处理器
     *
     * @param id 微信公众号配置ID
     * @return 微信公众号处理器
     */
    public static WechatPublicHandler getWechatPublicHandler(@NonNull String id) {
        WechatPublicHandler handler = getWechatPublicHandlers().get(id);
        return AssertUtils.nonnull(handler, () -> "Wechat public handler not found: " + id);
    }

    /**
     * 将键/值对转换成xml
     *
     * @param map 键/值对
     * @return xml
     */
    public static String map2xml(@NonNull Map<String, String> map) {
        try {
            return WXPayUtil.mapToXml(map);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将xml转换成键/值对
     *
     * @param xml XML
     * @return 键/值对
     */
    public static Map<String, String> xml2map(@NonNull String xml) {
        try {
            return WXPayUtil.xmlToMap(xml);
        } catch (Exception e) {
            throw new RuntimeException();
        }
    }

    /**
     * 参数签名
     *
     * @param key        签名密钥
     * @param parameters 签名参数
     * @return 签名信息
     */
    public static String sign(@NonNull String key, @NonNull Map<String, String> parameters) {
        try {
            return WXPayUtil.generateSignature(parameters, key);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 解码手机号
     *
     * @param ciphertext 会话信息密文
     * @param secret     密钥
     * @param iv         偏移量
     * @return 手机号明文
     */
    public static String mobile(String ciphertext, @NonNull String secret, @NonNull String iv) {
        if (StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }
        Cipher cipher = AESUtils.getDecrypter(Base64.decodeBase64(secret), Base64.decodeBase64(iv));
        Map<?, ?> map = JacksonUtils.deserialize(AESUtils.decrypt(cipher, ciphertext), Map.class);
        return map == null ? null : (String) map.get("phoneNumber");
    }

    /**
     * 数据解密
     *
     * @param ciphertext 密文
     * @param secret     密钥
     * @return 明文
     */
    public static String decrypt(String ciphertext, @NonNull String secret) {
        if (StringUtils.isEmpty(ciphertext)) {
            return ciphertext;
        }

        // 解密数据
        Cipher cipher = AESUtils.lookupDecrypter(AESUtils.CBC_NOPADDING + secret, () -> {
            byte[] bytes = secret.getBytes(StandardCharsets.UTF_8);
            byte[] iv = Arrays.copyOfRange(bytes, 0, 16);
            return AESUtils.getCipher(AESUtils.CBC_NOPADDING, Cipher.DECRYPT_MODE, bytes, iv);
        });
        byte[] original = AESUtils.codec(cipher, Base64.decodeBase64(ciphertext));

        // 删除解密后明文的补位字符
        int pad = original[original.length - 1];
        if (pad < 1 || pad > 32) {
            pad = 0;
        }
        byte[] bytes = Arrays.copyOfRange(original, 0, original.length - pad);

        // 分离16位随机字符串,网络字节序和AppId
        int length = 0;
        for (int i = 16; i < 20; i++) {
            length <<= 8;
            length |= bytes[i] & 0xff;
        }
        return new String(Arrays.copyOfRange(original, 20, 20 + length), StandardCharsets.UTF_8);
    }

    /**
     * 执行接口请求
     *
     * @param function 请求回调函数
     * @param <T>      结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T execute(@NonNull Function<RestTemplate, T> function) {
        T response = RestContextHolder.execute(function);
        int status = ObjectUtils.ifNull(response.getErrcode(), 0);
        if (status != 0) {
            throw new IllegalStateException(status + ", " + response.getErrmsg());
        }
        return response;
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T get(@NonNull String url, @NonNull Class<T> type,
                                                   @NonNull Object... parameters) {
        return execute(rest -> rest.getForObject(url, type, parameters));
    }

    /**
     * GET方式请求
     *
     * @param url        资源地址
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T get(@NonNull String url, @NonNull Class<T> type,
                                                   @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.getForObject(url, type, parameters));
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                                                    @NonNull Object... parameters) {
        return execute(rest -> rest.postForObject(url, request, type, parameters));
    }

    /**
     * POST方式请求
     *
     * @param url        资源地址
     * @param request    请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T post(@NonNull String url, Object request, @NonNull Class<T> type,
                                                    @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.postForObject(url, request, type, parameters));
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity, @NonNull Class<T> type,
                                                        @NonNull Object... parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity, @NonNull Class<T> type,
                                                        @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity,
                                                        @NonNull ParameterizedTypeReference<T> type,
                                                        @NonNull Object... parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }

    /**
     * 接口调用
     *
     * @param url        资源地址
     * @param method     请求方式
     * @param entity     请求实体
     * @param type       对象类型
     * @param parameters 请求参数
     * @param <T>        结果类型
     * @return 微信响应结果
     */
    public static <T extends WechatResponse> T exchange(@NonNull String url, @NonNull HttpMethod method,
                                                        HttpEntity<?> entity,
                                                        @NonNull ParameterizedTypeReference<T> type,
                                                        @NonNull Map<String, ?> parameters) {
        return execute(rest -> rest.exchange(url, method, entity, type, parameters).getBody());
    }
}
